var TLAutoPlayController = function(data) {
	var self = this;
	self.controller = data.controller;
	self.timeline = self.controller.timeline;
	self.contentPanelController = self.controller.contentPanelController;
	self.startTimeoutDelay = 1000;
	self.delayBetweenTimelineScrolls = 2000;
	self.scrollToStartSpeed = 1500;
	self.paused = true;
	self.pauseEventTookPlace = 0;
	self.lastMouseCoords = {x: 0, y: 0}
}

TLAutoPlayController.prototype = {
	init: function() {
		var self = this;
		self.initSettings();
		return self;	
	},
	initSettings: function() {
		var self = this;
		var autoPlay = self.timeline.autoPlay;
		self.timeline.autoPlay = (!autoPlay || !autoPlay.split || autoPlay.split(",").length !=3) ? "0,5,5" : autoPlay;
		
		var aSets = self.timeline.autoPlay.split(",");
		
		self.timeline.autoPlaySettings = {
			mode: aSets[0], // 0 = off, 1 = scroll through timeline, 2 = scroll between stories, 3 = scroll between stories and open story panels,
			scrollSpeed: aSets[1], // 1 - 10 slow to faster
			storyPause: aSets[2] // 1 - 10 shorter to longer
		}
		self.settings = self.timeline.autoPlaySettings;
	},
	activateOverlay: function() {
		var self = this;
		if (!self.domOverlay) {
			self.domOverlay = $('<div class="tl-auto-play-overlay"></div>').get()[0];
		}

		$(self.domOverlay).mousedown( function() {
			self.pauseAutoPlay();
		});

		var vSize = AJKHelpers.viewportSize();
		
		$(self.domOverlay).css({ width: vSize.width, height: vSize.height });
		
		$("body").append(self.domOverlay);
		
	},
	deactivateOverlay: function() {
		var self = this;
		if (self.domOverlay) {
			$(self.domOverlay).unbind("mousedown").remove();
		}
	},
	setup: function() {
		var self = this;
		if (self.settings.mode == 0) {
			self.disableAutoPlay();
		}
		else {
			self.enableAutoPlay();
		}	
	},
	reset: function() {
		var self = this;
		self.disableAutoPlay();
		self.setup();
	},
	disableAutoPlay: function() {
		var self = this;	
		self.stopAutoPlay();
		if (self.pauseTimeout) {
			clearTimeout(self.pauseTimeout);
			self.pauseTimeout = "";
		}
		theAJKMouseMoveEvent.removeAsObserver({ observer: self });
		theAJKKeyEvent.removeAsObserver({ observer: self });
	},
	pauseAutoPlay: function() {
		var self = this;
		
		theAJKMouseScrollEvent.removeAsObserver({ observer: self });
		
		if (!self.paused) {
			self.paused = true;
			
			if (self.pauseTimeout) {
				clearTimeout(self.pauseTimeout);
			}
			self.stopAutoPlay();
			
			var clearCounter = 0;
			(function() {
				var thisFunc = arguments.callee;
				self.pauseTimeout = setTimeout( function() {
					clearCounter++;
					if (self.pauseEventTookPlace || self.contentPanelController.panelVisible) {
						self.pauseEventTookPlace = false;
						clearCounter = 0;
					}
					if (clearCounter >= 5 && !self.contentPanelController.panelVisible) {
						self.pauseTimeout = "";
						self.startAutoPlay();
					}
					else {
						thisFunc();
					}
				},1000);
				
			})(); 
		}
		else {
			self.pauseEventTookPlace = true;
		}

	},
	enableAutoPlay: function() {
		var self = this;
		self.stopAutoPlay();
		self.paused = false;
		
		theAJKMouseMoveEvent.registerAsObserver({ observer: self });
		theAJKKeyEvent.registerAsObserver({ observer: self });

		self.timeout = setTimeout( function() {
			self.startAutoPlay();
		},self.startTimeoutDelay);
	},
	keyEventTookPlace: function(data) {
		var self = this;
		self.pauseAutoPlay();
	},
	mouseDidMove: function(data) {
		var self = this;
		var coords = data.coords;
		
		// detect that the coords arethe same
		// Prevents issue from random mousedidmove events firing
		if (!(coords.x == self.lastMouseCoords.x && coords.y == self.lastMouseCoords.y)) {
			self.pauseAutoPlay();
		}
		
		self.lastMouseCoords.x = coords.x;
		self.lastMouseCoords.y = coords.y;
		
	},
	mouseDidScroll: function() {
		var self = this;
		self.pauseAutoPlay();
	},
	stopAutoPlay: function() {
		var self = this;	

		self.cancelStoryLoading = true;
		
		if (self.timeout) {
			clearTimeout(self.timeout);
			self.timeout = "";
		}
		if (self.animation) {
			$(self.animation).stop();
			self.animation = "";	
		}
		
		if (theTLSettings.dateAnimation) {
			$(theTLSettings.dateAnimation).stop();
			theTLSettings.dateAnimation = "";	
		}
		
		self.deactivateOverlay();
	},
	startAutoPlay: function() {
		var self = this;
		theAJKMouseScrollEvent.registerAsObserver({ observer: self });
		self.paused = false;
		self.activateOverlay();
		if (self.settings.mode == 1) {
			self.startTimelineScroll();
		}
		else if (self.settings.mode == 2 || self.settings.mode == 3) {
			self.startStoryScroll();
		}
	},
	startStoryScroll: function() {
		var self = this;
		var storyInfo = {};
		var direction = "next";
		var storyDelayPause = 1000 * self.settings.storyPause;
		var storyScrollSpeed = 500 * self.settings.scrollSpeed;
		if (self.controller.markers.length == 0) {
			return;
		}
		
		var selectedStory = theTLSettings.lastSelectedStory;
		
		if (!selectedStory) {
			var stories = self.controller.getNextPreviousStoriesFromCurrentPos();
			selectedStory = stories.closestStory;	
		}
		var thisCode = (new Date()).getTime();

		var self = this;
		( function() {
			var speed = (direction == "first") ? self.scrollToStartSpeed : storyScrollSpeed;
			var thisFunc = arguments.callee;
						
			self.controller.focusMarker({ 
				marker: selectedStory,
				speed: speed, 
				callback: function() {
					var thisStory = selectedStory;
					
					selectedStory = self.controller.markers[selectedStory.orderIndex+1];
					if (!selectedStory) {
						selectedStory = self.controller.markers[0];
						direction = "first";
					}
					else {
						direction = "next";	
					}
					
					if (self.settings.mode == "3") {
						self.loadingStoryContent = true;
						self.cancelStoryLoading = false;
						
						thisStory.loadExtraInfo({
							callback: function() {
								self.loadingStoryContent = false;

								if (self.cancelStoryLoading) {
									self.cancelStoryLoading = false;
									return;
								}
								var delay = storyDelayPause + 500+10; // 500 = time to fade in story panel
								
								if (!(self.contentPanelController.panelVisible && thisStory == self.contentPanelController.selectedStory)) {
									// Open and close story panel before scrolling to the next option
									self.contentPanelController.displayForStory({ story: thisStory });
								}
																
								self.timeout = setTimeout( function() {
									self.contentPanelController.hide();
									self.timeout = setTimeout( function() {
										thisFunc();
									},510);
									
								},storyDelayPause);
							}
						});
						
					}
					else {
						// Just scroll between stories or back to the start of the timeline
						self.timeout = setTimeout( function() {
							thisFunc();
						},storyDelayPause);
					}					
				}
			});
		})();
	},
	startTimelineScroll: function() {
		var self = this;
		
		var startDate = self.controller.originalStartDate;
		var endDate = self.controller.originalEndDate;
		
		if (self.timeline.markerSpacing == "equal") {
			var stories = self.controller.markers;
			var firstStory = stories[0];
			if (firstStory) {
				startDate = AJKHelpers.cloneDate({ date: firstStory.fakeStartDate });
			}

			var lastStory = stories[stories.length-1];
			if (lastStory) {
				endDate = AJKHelpers.cloneDate({ date: lastStory.fakeStartDate });
			}
		}
						
		var pLength = self.controller.selectedView.width;
		var tLength = endDate.getTime()-startDate.getTime();
		tLength = (tLength < 1) ? 1 : tLength;
		var tPos = theTLSettings.currentDate.getTime()-startDate.getTime();
		tPos = (tPos < 1) ? 1 : tPos;
		var pPos = tPos/tLength * pLength;
		var scrollDist = pLength-pPos;
		var scrollTime = Math.round((scrollDist / 500) * Math.pow(self.settings.scrollSpeed,2) * 100);
		var scrollDate = new Date();
				
		self.animation = $({ time: theTLSettings.currentDate.getTime() }).animate({
			time: endDate.getTime()
		},{
			duration: scrollTime,
			easing: "linear",
			step: function() {
				scrollDate.setTime(Math.round(this.time));
				
				scrollDate = theTLSettings.limitDateToRange({aDate: scrollDate });
								
				theTLSettings.setCurrentDate({
					date: scrollDate,
					instantly: true,
					extraInfo: {
						updateHash: false
					}
				});
			},
			complete: function() {

				self.timeout = setTimeout( function() {
					
					var tDate = theTLSettings.limitDateToRange({aDate: startDate });
					
					theTLSettings.setCurrentDate({ 
						date: tDate,
						animate: true,
						speed: self.scrollToStartSpeed,
						callback: function() {
							
							self.timeout = setTimeout( function() {
								self.startTimelineScroll();
							},self.delayBetweenTimelineScrolls);
							
							
						}
					});
				},self.delayBetweenTimelineScrolls)
				
			}
		});	

			
	}
}