var TLSpanController = function(data) {
	var self = this;
	self.spanData = data.spanData || [];
	self.mainController = data.mainController;
	self.spans = [];
	self.spansByKey = [];
	self.domBackgroundHolder = $("#tl-stage-span-backgrounds").get()[0];
	self.domSliderHolder = $("#tl-slider-spans-holder").get()[0];
	self.domBlockHolder = self.mainController.selectedView.domSpanBlocksHolder;
	self.currentSpan = "";
}

TLSpanController.prototype = {
	init: function() {
		var self = this;

		theTLSettings.registerAsObserver({
			type: "currentDate",
			observer: self
		});

		theAJKWindowResizeEvent.registerAsObserver({ observer: self });

		var imgDelay = 0;
		var imgCounter=1;
		var imgDelayInc = 500;
		var imgIncNum = 5;
		$.each( self.spanData, function() {
			if (imgCounter == imgIncNum) {
				imgCounter=1;
				imgDelay+=imgDelayInc;
			}
			if (this.image) {
				imgCounter++;
			}
			this.imageLoadDelay = imgDelay;
			self.createNewSpan(this);
		});
		self.dealWithMainImageCaption();
		self.sortSpans();
		self.updateSpansInDom();
		
		//self.spans[0].displayBackground();
		
		return self;
	},
	refreshSpansAfterViewChange: function() {
		var self = this;
		// This is called when we switch from 2d to 3d and vice versa
		$.each(self.spans, function() {
			if (this.style == 2) {
				this.updateStyle();
				this.resizeIfNeeded();
				this.refresh();	
			}
		});
		
		var cSpan = self.getBackgroundSpanForDate({ date: theTLSettings.currentDate });
		if (cSpan && cSpan != self.currentSpan) {
			if (self.currentSpan) {
				self.currentSpan.hideBackground();
			}
			cSpan.displayBackground();
			self.currentSpan = cSpan;
		} 
	},
	dealWithMainImageCaption: function() {
		var self = this;
		if (self.spans.length > 0) {
			$("#tl-stage-main-photo-credit").css({ "z-index": "auto" });
		}
		else {
			$("#tl-stage-main-photo-credit").css({ "z-index": 1 });
		}
	},
	repositionBlockContent: function() {
		var self = this;
		$.each(self.spans, function() {
			this.repositionBlockContentIfVisible();
		});
	},
	updateSpanDates: function() {
		var self = this;
		$.each(self.spans, function() {
			this.updateDates();
		});
	},
	updateSpansInDom: function() {
		var self = this;
		$.each(self.spans, function() {
			$(self.domBackgroundHolder).append(this.background.domRoot);
			$(self.domSliderHolder).append(this.slider.domRoot);
			$(self.domBlockHolder).append(this.block.domRoot);
		});
	},
	viewWasUpdated: function() {
		var self = this;
		// Need to resize all the slider blocks for spans, and add them back to the slider
		self.domBlockHolder = self.mainController.selectedView.domSpanBlocksHolder;

		$.each(self.spans, function() {
			$(self.domBlockHolder).append(this.block.domRoot);
			this.updateDatePositions();
		});
	},
	windowDidResize: function(data) {
		var self = this;
		
		$.each(self.spans, function() {
			this.resizeIfNeeded();
		});
	},
	sortSpans: function() {
		var self = this;
		var i = 0;
		self.spans.sort(function(a,b) {
			return (a.start.getTime() > b.start.getTime()) ? 1 : -1; 
		});
		$.each(self.spans, function() {
			this.orderIndex = i++;
		});
			
	},
	getBackgroundSpanForDate: function(data) {
		var self = this;
		var aDate = data.date;
		var sSpan = false;
		
		var vContr = theTLMainController.selectedView;
		var datePos = -vContr.getLeftOffsetForDate({ date: aDate });
				
		$.each(self.spans, function() {
			if (this.getEffectiveStyle() >= 2) {
				// Only applies to background spans, not block spans
				return;	
			}
						
			if (this.block.startPos > datePos ) {
				return false; // exit loop	
			}
			else if (datePos >= this.block.startPos && datePos <= this.block.endPos) {
				sSpan = this;
			}
			
			/*if (this.start.getTime() > aDate.getTime() ) {
				return false; // exit loop	
			}
			else if (aDate.getTime() >= this.start.getTime() && aDate.getTime() <= this.end.getTime()) {
				sSpan = this;
			}*/
		});
		return sSpan;
	},
	currentDateWasUpdatedTo: function(data) {
		var self = this;
		
		if (self.switchTimeout) {
			clearTimeout(self.switchTimeout);
			self.switchTimeout = false;
		}
		
		if (self.animating) {
			var thisFunc = arguments.callee;
			self.switchTimeout = setTimeout( function() {
				self.currentDateWasUpdatedTo(data)
			},100);
			return;
		}

		var span = self.getBackgroundSpanForDate({ date: data.date });
		
		
		if (span != self.currentSpan) {
			self.animating = true;
			if (!self.currentSpan) {
				span.fadeBackgroundIn({
					callback: function() {
						self.animating = false;	
					}	
				});
			}
			else if (span && self.currentSpan) {
				span.fadeBackgroundUnderSpan({ span: self.currentSpan });
				self.currentSpan.fadeBackgroundOut({
					callback: function() {
						self.animating = false;	
					}
				});
			}
			else if (!span && self.currentSpan) {
				self.currentSpan.fadeBackgroundOut({
					callback: function() {
						self.animating = false;	
					}
				});
			}
			self.currentSpan = span;
		}
	},
	refresh: function() {
		var self = this;
		self.currentDateWasUpdatedTo({ date: theTLSettings.currentDate });	
	},
	createNewSpan: function(data) {
		var self = this;
		data.controller = self;
		if (!data.imageDataUri) {
			data.imageDataUri = "";
		}
		var newSpan =  new TLSpan(data).init();
		self.spans.push(newSpan);
		self.spansByKey[newSpan.key] = newSpan;
		return newSpan;
	}
}


var TLSpan = function(data) {
	var self = this;
	self.id = data.id;
	self.key = "key-"+self.id;
	self.start = AJKHelpers.dateFromMySQLDate({ dateString: data.start });	
	self.end = AJKHelpers.dateFromMySQLDate({ dateString: data.end });
	self.title = data.title;
	self.image = data.image;
	self.imageDataUri = data.imageDataUri;
	self.style = data.style;
	self.background = {};
	self.slider = {};
	self.block = {};
	self.color = data.color;
	self.textColor = data.textColor;
	self.animTime = 500;
	self.imageLoaded = false;
	self.colorOpacity = data.colorOpacity;
	self.showInSlider = data.showInSlider;
	self.showText = data.showText;
	self.imageCredit = data.imageCredit;
	self.controller = data.controller;
	self.imageInitialised = false;
	self.imageLoadDelay = (data.imageLoadDelay) ? data.imageLoadDelay : 0;
	self.imagePositioning = 0;
	if (TLTimelinePrintMode) {
		self.imagePositioning = 1;
	}

}

TLSpan.prototype = {
	init: function() {
		var self = this;

		self.initBackground();
		self.initSlider();
		self.initBlock();
		self.update();
		
		self.resizeIfNeeded();
		
		if (self.image && self.imageLoadDelay == 0) {
			self.initImage();	
		}
		else {
			setTimeout( function() {
				self.initImage();	
			},self.imageLoadDelay);
		}
		
		self.refresh();
		
		return self;	
	},
	getEffectiveStyle: function() {
		var self = this;
		if (self.style == 2 && theTLMainController.selected3DView.active) {
			return 1;	
		}
		else {
			return self.style;
		}
	},
	getEffectiveImagePositioning: function() {
		var self = this;
		return self.imagePositioning;
	},
	isBlockVisible: function() {
		var self = this;
		var vContr = theTLMainController.selectedView;
		var distFromLeft = self.block.startPos + vContr.realStageLeftOffset;
		var distFromRight = self.block.endPos + vContr.realStageLeftOffset;
		var isVis = (distFromRight > 0 && distFromLeft < theTLSettings.windowSize.width);
		return isVis;
	},
	currentDateWasUpdatedTo: function(data) {
		var self = this;
		var sliderDrag = (data.extraInfo && data.extraInfo.drag); 
		// This is onlu called when we are in style 2
		if (!sliderDrag) {
			self.repositionBlockContentIfVisible();
		}
	},
	resizeIfNeeded: function() {
		var self = this;
		if (self == self.controller.currentSpan) {
			self.resize();	
		}
		else if (self.getEffectiveStyle() == 2) {
			if (self.isBlockVisible()) {
				self.resize();
				self.resizeNeeded = false;
			}
			else {
				self.resizeNeeded = true;
			}
		}
	},
	repositionBlockContentIfVisible: function() {
		var self = this;
		if (self.getEffectiveStyle() != 2) { return }
		if (self.isBlockVisible()) {
			if (self.image && !self.imageInitialised) {
				self.initImage();	
			}
			self.repositionBlockContent();
		}
		else if (self.lastBlockTextStyle == 3) {
			self.unFixPositionBlockContent();
		}
	},
	unFixPositionBlockContent: function() {
		var self = this;
		$(self.block.domContent).css({ position: "absolute" });
		self.lastBlockTextStyle == 0;
	},
	repositionBlockContent: function() {
		var self = this;
		if (self.resizeNeeded) {
			self.resize();
			self.resizeNeeded = false;	
		}

		if (self.imageLoaded) {
			self.positionBlockImage();
		}
		self.positionBlockText();

	},
	positionBlockImage: function() {
		var self = this;
		if (self.getEffectiveImagePositioning() == 1) {
			$(self.block.domImageHolder).css({ left: 0 });
		}
		else {
			var vContr = theTLMainController.selectedView;
			var stageOffset = (vContr.realStageLeftOffset >= 0) ? Math.round(vContr.realStageLeftOffset) : -Math.round(-vContr.realStageLeftOffset);
			var distFromLeft = self.block.startPos + stageOffset;
			$(self.block.domImageHolder).css({ left: -distFromLeft });
		}
	},
	positionBlockText: function() {
		var self = this;
		var vContr = theTLMainController.selectedView;
		var distFromLeft = self.block.startPos + vContr.realStageLeftOffset;
		var distFromRight = self.block.endPos + vContr.realStageLeftOffset;
		var hPad = 15;
		var vPad = 10;
		
		if (self.block.contentWidth == 0 && self.showText != 1) {
			self.block.contentWidth = $(self.block.domContent).width();
		}

				
		if (distFromLeft > 0 || ((distFromRight-distFromLeft) <= (self.block.contentWidth+ 2 * hPad))) {
			if (self.lastBlockTextStyle != 1) {
				$(self.block.domContent).css({ left: hPad, right: "auto", position: "absolute", top: vPad, "z-index": "auto" });
				self.lastBlockTextStyle = 1;
			}
		}
		else if (distFromRight < (self.block.contentWidth+ 2 * hPad)) {
			if (self.lastBlockTextStyle != 2) {
				$(self.block.domContent).css({ left: "auto", right: hPad, position: "absolute", top: vPad, "z-index": "auto" });
				self.lastBlockTextStyle = 2;
			}
		}
		else {
			if ($.browser.isIE6) {
				$(self.block.domContent).css({ left: hPad -distFromLeft, right: "auto",position: "absolute", top: vPad }); // may need to use this in old MSs
			}
			else {
				var t = theTLMainController.timeline;
				var	extraHeight = ((t.showTitleBlock || t.embed == "false") && !theTLMainController.minimumLayout) ? 36 : 0;
				if (self.lastBlockTextStyle != 3 || self.lastBlockTextExtraHeight != extraHeight) {
					$(self.block.domContent).css({ left: hPad, right: "auto", position: "fixed", top: vPad + extraHeight, "z-index": 1 }); // zindex needed for Safari to prevent bg image stutter
					self.lastBlockTextExtraHeight = extraHeight;
					self.lastBlockTextStyle = 3;
				}
			}
		}
				
	},
	initBlock: function() {
		var self = this;	
		var iH = '<div id="tl-span-block-'+self.key+'" class="tl-span-block">';
		iH += '<div class="tl-span-block-image-holder"></div>';
		iH += '<div class="tl-span-block-color">';
		iH += '<div class="tl-span-block-border1"></div>';
		iH += '<div class="tl-span-block-border2"></div>';
		iH += '</div>';
		iH += '<div class="tl-span-content">';
		iH += '<h4 class="tl-font-head"></h4>';
		iH += '<h5></h5>';
		iH += '<div class="tl-span-credit"></div>';
		iH += '</div></div>';
		var bl = self.block;
		bl.domRoot = $(iH).get()[0];
		bl.domColor = $(bl.domRoot).find(".tl-span-block-color").get()[0];  
		bl.domTitle = $(bl.domRoot).find("h4").get()[0];  
		bl.domDates = $(bl.domRoot).find("h5").get()[0]; 
		bl.domContent = $(bl.domRoot).find(".tl-span-content").get()[0]; 
		bl.domImageHolder = $(bl.domRoot).find(".tl-span-block-image-holder").get()[0]; 
		bl.domImageCredit = $(bl.domRoot).find(".tl-span-credit").get()[0]; 
	},
	initSlider: function() {
		var self = this;
		var iH = '<div id="tl-span-slider-'+self.key+'" class="tl-span-slider">';
		iH += '<h4>Title</h4>';
		iH += '<div class="tl-span-block-color">';
		iH += '<div class="tl-span-block-border1"></div>';
		iH += '<div class="tl-span-block-border2"></div>';
		iH += '</div>';
		iH += '</div>';
		self.slider.domRoot = $(iH).get()[0];
		self.slider.domColor = $(self.slider.domRoot).find(".tl-span-block-color").get()[0];  
		self.slider.domTitle = $(self.slider.domRoot).find("h4").get()[0];  
	},
	initBackground: function(data) {
		var self = this;
		var iH = '<div id="tl-span-background-'+self.key+'" class="tl-span-background">';
		iH += '<div class="tl-span-color"></div>';
		iH += '<div class="tl-span-content">';
		iH += '<h4 class="tl-font-head"></h4>';
		iH += '<h5></h5>';
		iH += '<div class="tl-span-credit"></div>';
		iH += '</div>';
		iH += '<div class="tl-span-credit tl-span-credit-2"></div>';
		iH += '</div>';
		var bg = self.background;
		bg.domRoot = $(iH).get()[0];
		bg.domColor = $(bg.domRoot).find(".tl-span-color").get()[0];  
		bg.domTitle = $(bg.domRoot).find("h4").get()[0];  
		bg.domDates = $(bg.domRoot).find("h5").get()[0]; 
		bg.domContent = $(bg.domRoot).find(".tl-span-content").get()[0]; 
		bg.domImageCredit = $(bg.domRoot).find(".tl-span-credit").get()[0]; 
		bg.domImageCredit2 = $(bg.domRoot).find(".tl-span-credit-2").get()[0]; 
	},
	update: function() {
		var self = this;
		self.updateColor();
		self.updateColorOpacity();
		self.updateTitle();
		self.updateDates();
		self.updateImageCredit();
		self.updateTextColor();
		self.updateStyle();
		self.updateShowText();
		self.updateShowInSlider();
	},
	updateShowInSlider: function() {
		var self = this;
		if (self.showInSlider == 1) {
			$(self.slider.domRoot).css({ display: "block" });
		}
		else {
			$(self.slider.domRoot).css({ display: "none" });
		}
	},
	initImage: function() {
		var self = this;
		if (self.imageInitialised) {
			return;	
		}
		self.imageInitialised = true;
		var bg = self.background;
		var bl = self.block;
		self.imageLoaded = false;
		var imageUrl = (self.imageDataUri) ? self.imageDataUri : self.image;
		bg.domImage = $('<img src="'+imageUrl+'" alt="'+$(bg.domTitle).text()+'" class="tl-span-background-image" />').get()[0];
		self.imageIsLoading = true;
		self.imageLoader = new AJKImageLoader({
			imageUrl: imageUrl,
			loadCallback: function(data) {
				setTimeout( function() {
					$("#tl-image-preloader").prepend(bg.domImage);
					self.imageSize = {
						width: bg.domImage.naturalWidth || $(bg.domImage).width(),
						height: bg.domImage.naturalHeight || $(bg.domImage).height()	
					}
					self.imageLoaded = true;
					bl.domImage = $(bg.domImage).clone().get()[0];
					self.resize();
					$(bg.domRoot).prepend(bg.domImage);
					$(bl.domImageHolder).prepend(bl.domImage);
					self.refresh();
					self.imageIsLoading = false;
					self.imageLoader = "";
				},1);
			}
		}).init();
	},
	updateImage: function() {
		var self = this;
		self.imageInitialised = false;
		if (self.background.domImage) {
			$(self.background.domImage).remove();
		}
		if (self.block.domImage) {
			$(self.block.domImage).remove();
		}
		if (self.imageLoader) {
			self.imageLoader.cancelImageCallback();
			self.imageLoader = "";
		}
		if (self.image) {
			self.initImage();	
		}
	},
	updateStyle: function() {
		var self = this;
		
		var bg = self.background;
		var bl = self.block;
		
		theTLSettings.removeAsObserver({
			observer: self
		});
			
		if (self.getEffectiveStyle() == 0) {
			$([bg.domColor,bl.domRoot]).css({ display: "none" });
			if (self == self.controller.currentSpan) {
				$([bg.domRoot]).css({ display: "block" }); 
			}
		}
		else if (self.getEffectiveStyle() == 1) {
			$(bl.domRoot).css({ display: "none" });  
			$([bg.domColor,bg.domContent]).css({ display: "block" }); 
			if (self == self.controller.currentSpan) {
				$([bg.domRoot]).css({ display: "block", opacity: 1 }); 
			}
		}
		else {
			$([bg.domColor,bg.domContent, bg.domRoot]).css({ display: "none" }); 
			$(bl.domRoot).css({ display: "block" });
			theTLSettings.registerAsObserver({
				type: "currentDate",
				observer: self
			});
		}
	},
	updateTextColor: function() {
		var self = this;
		$([self.background.domDates,self.background.domTitle,self.background.domImageCredit,self.block.domImageCredit,self.block.domDates,self.block.domTitle,self.slider.domTitle]).css({ color: "#" + self.textColor });
		$(self.background.domImageCredit).find("a").css({ color: "#" + self.textColor });
		$(self.block.domImageCredit).find("a").css({ color: "#" + self.textColor });
	},
	updateColor: function() {
		var self = this;
		$([self.background.domColor,self.slider.domColor,self.block.domColor]).css({ backgroundColor: "#"+self.color });
	},
	updateColorOpacity: function() {
		var self = this;	
		$([self.background.domColor,self.block.domColor]).css({ opacity: self.colorOpacity/100 });
		//$(self.slider.domRoot).css({ opacity: self.colorOpacity/100 });
	},
	updateImageCredit: function() {
		var self = this;	
		$([self.background.domImageCredit,self.background.domImageCredit2,self.block.domImageCredit]).html(self.imageCredit);
	},
	updateTitle: function() {
		var self = this;	
		$([self.background.domTitle,self.block.domTitle,self.slider.domTitle]).html(self.title);
	},
	updateDates: function() {
		var self = this;
		var format = theTLMainController.timeline.topDateFormat;
		format = (!format || format == "auto") ? TLViewController.viewScaleSettings[theTLMainController.selectedView.type].displayDateFormat : format;
		format = (!format) ? "ddnn MMMM YYYY HH:00" : format;
		
		$([self.background.domDates,self.block.domDates]).html(AJKHelpers.formatDate({ date: self.start, formatString: format })+" - "+AJKHelpers.formatDate({ date: self.end, formatString: format }));
		self.updateDatePositions();
	},
	updateDatePositions: function() {
		var self = this;
		
		var vContr = theTLMainController.selectedView;
		if (theTLMainController.timeline.markerSpacing == "equal") {
			var startDatePos = vContr.getEquallySpacedOffsetForDate({ aDate: self.start });
			var endDatePos = vContr.getEquallySpacedOffsetForDate({ aDate: self.end });
			startDatePos = (startDatePos == "-1") ? 0 : startDatePos;
			endDatePos = (endDatePos == "-1") ? 0 : endDatePos;
			pStart = startDatePos/vContr.width * 100;
			pEnd = endDatePos/vContr.width * 100;
		}
		else {
			var sContr = theTLMainController.sliderController;
			var pStart = sContr.getLeftOffsetPercentForDate({ date: self.start });
			var pEnd = sContr.getLeftOffsetPercentForDate({ date: self.end });
		}
		pStart = (pStart < 0) ? 0 : pStart;
		pStart = (pStart > 100) ? 100 : pStart;
		pEnd = (pEnd < 0) ? 0 : pEnd;
		pEnd = (pEnd > 100) ? 100 : pEnd;
		$(self.slider.domRoot).css({ left: pStart+"%", width: (pEnd-pStart)+"%" });
		self.block.startPos = Math.round(vContr.width * pStart/100,10);
		self.block.endPos = Math.round(vContr.width * pEnd/100,10);
		$(self.block.domRoot).css({ left: self.block.startPos, width: self.block.endPos-self.block.startPos });
	},
	updateShowText: function() {
		var self = this;
		if (self.showText == 1) {
			$([self.background.domContent,self.block.domContent,self.slider.domTitle]).css({ display: "none" });
		}
		else {
			$([self.background.domContent,self.block.domContent,self.slider.domTitle]).css({ display: "block" });
		}	
	},
	fadeBackgroundUnderSpan: function(data) {
		var self = this;
		if (self.getEffectiveStyle() == 2) {
			return;
		}
		self.resize();
		var span = data.span;
		if (span.getEffectiveStyle() != 2 && (span.imageLoaded || (span.getEffectiveStyle()==1 && span.colorOpacity == 100))) {
			$(span.background.domRoot).before(self.background.domRoot);
			$(self.background.domRoot).css({ display: "block", opacity: 1 });
		}
		else {
			self.fadeBackgroundIn();
		}
	},
	fadeBackgroundIn: function(data) {
		var self = this;
		var callback = (data) ? data.callback : "";
		if (self.getEffectiveStyle() == 2) {
			if (callback) { callback(); }
			return;
		}
	
		if (self.image && !self.imageInitialised) {
			self.initImage();	
		}
		self.resize();
		//console.log("fadein");
		$(self.background.domRoot)
		.css({ opacity: 0, display: "block" })
		.animate({ opacity: 1	}, self.animTime, function() {
			if (callback) { callback(); }
		});
	},
	fadeBackgroundOut: function(data) {
		var self = this;
		var callback = (data) ? data.callback : "";
		if (self.getEffectiveStyle() == 2) {
			if (callback) { callback(); }
			return;
		}
		//console.log("fadeout");
		$(self.background.domRoot)
		.animate({ opacity: 0	}, self.animTime, function() {
			$(this).css({ display: "none" });
			if (callback) { callback(); }
		});
	},
	displayBackground: function() {
		var self = this;
		$(self.background.domRoot).css({ display: "block", opacity: 1 });	
	},
	hideBackground: function() {
		var self = this;
		$(self.background.domRoot).css({ display: "none" });	
	},
	resize: function() {
		var self = this;

		if (self.imageLoaded) {
			if (self.getEffectiveStyle() == 2) {
				var blockWidth = self.block.endPos - self.block.startPos;
				if (self.getEffectiveImagePositioning() == 1) {
					// Image fits the size of the block
					$(self.block.domImageHolder).css({ width: blockWidth, height: theTLMainController.viewContentSize.height });
					AJKHelpers.sizeImageToFitInBoxOfSize({
						domImage: self.block.domImage,
						boxSize: {
							width: blockWidth,
							height: theTLMainController.viewContentSize.height
						},
						imageSize: self.imageSize
					});
				}
				else {
					// Image fits screensize
					$(self.block.domImageHolder).css({ width: theTLMainController.viewContentSize.width, height: theTLMainController.viewContentSize.height });
					AJKHelpers.sizeImageToFitInBoxOfSize({
						domImage: self.block.domImage,
						boxSize: {
							width: theTLMainController.viewContentSize.width,
							height: theTLMainController.viewContentSize.height
						},
						imageSize: self.imageSize
					});
				}
			}
			else {
				AJKHelpers.sizeImageToFitInBoxOfSize({
					domImage: self.background.domImage,
					boxSize: {
						width: theTLMainController.viewContentSize.width,
						height: theTLMainController.viewContentSize.height
					},
					imageSize: self.imageSize
				});
			}
			
		}

		if (self.getEffectiveStyle() == 2) {
			self.block.contentWidth = $(self.block.domContent).width();
		}

	},
	refresh: function() {
		var self = this;
		self.currentDateWasUpdatedTo({ date: theTLSettings.currentDate });	
	},
	destroy: function() {
		var self = this;
		// Remove completely from dom
		$(self.background.domRoot).remove();
		$(self.slider.domRoot).remove();
		$(self.block.domRoot).remove();
		
		self.background = "";
		self.block = "";
		self.slider = "";
		// Remove block
		// Remove slider block too	
	}
}










