/* Copyright 2013 Webalon Ltd
 *
 * It is strictly forbidden to copy, distribute, modify or reuse this file or its
 * contents without written permission from the copyright holder, Webalon Ltd
 *
 **************************************************************************************/

var TLYouTubeFeedHelper = {
	defautNumVideos: 20,
	defaultThumbImage: "/assets/ui/youtube-default-thumb.jpg",
	getVideosForFeed: function(data) {
		var feed = data.feed;
		var callback = data.callback;
		var numItems = (feed.numItems > 50) ? 50 : feed.numItems;
		
		if (feed.filter == "youtube-username") {
			TLYouTubeFeedHelper.getVideosFromUsername({
				numVideos: numItems,
				startIndex: 1,
				username: feed.param1,
				callback: function(data) {
					callback({
						videos: data.itemObjs	
					});
				}
			});
		}
		else if (feed.filter == "youtube-search") {
			TLYouTubeFeedHelper.getVideosForSearchTerm({
				numVideos: numItems,
				startIndex: 1,
				searchTerm: feed.param1,
				callback: function(data) {
					callback({
						videos: data.itemObjs	
					});
				}
			});
		}
		else {
			var feedType = feed.filter.split("youtube-")[1];
			TLYouTubeFeedHelper.getVideosForStandardFeedOfType({
				type: feedType,
				numVideos: numItems,
				startIndex: 1,
				callback: function(data) {
					callback({
						videos: data.itemObjs	
					});
				}
			});
		}
	},
	getVideosFromUsername: function(data) {
		var callback = data.callback;
		
		$.getJSON("https://www.googleapis.com/youtube/v3/channels",{
			part : 'contentDetails', 
			forUsername : data.username,
			key: TLYouTubeFeedHelper.apiKey
		},function(data) {
			if (data.items) {
				var item = data.items[0];
				if (!item) {
					return;	
				}
				var channelId = item.contentDetails.relatedPlaylists.uploads;
				if (channelId) {
					$.get( "https://www.googleapis.com/youtube/v3/playlistItems",{
						order: "date",
						part : "snippet", 
						maxResults: data.numVideos,
						maxResults : data.numVideos,
						playlistId : channelId,
						key: TLYouTubeFeedHelper.apiKey
					},function(data) {
						TLYouTubeFeedHelper.feedLoadFunc({
							callback: callback,
							feedData: data	
						});
					});
				}
			}
		});


		/*$.getJSON("https://gdata.youtube.com/feeds/api/users/"+data.username+"/uploads"+"?callback=?", {
			"start-index": data.startIndex,
			"max-results": data.numVideos,
			"v": 2,
			"alt": "json"
		}, function(data) {
			TLYouTubeFeedHelper.feedLoadFunc({
				callback: callback,
				feedData: data	
			}); 
		});*/
	},
	getVideosForStandardFeedOfType: function(data) {
		var callback = data.callback;
		if (data.type != "most_popular") {
			return;	
		}
		
		// Only offer most popular now
		$.getJSON("https://www.googleapis.com/youtube/v3/videos?part=snippet&chart=mostPopular&key="+TLYouTubeFeedHelper.apiKey+"&callback=?", {
			"start-index": data.startIndex,
			"maxResults": data.numVideos,
			"alt": "json",
			"order": "date"
		}, function(data) {
			TLYouTubeFeedHelper.feedLoadFunc({
				callback: callback,
				feedData: data	
			}); 
		});
	},
	getVideosForSearchTerm: function(data) {
		var callback = data.callback;
		
		
		$.getJSON("https://www.googleapis.com/youtube/v3/search",{
			part : "snippet", 
			order: "date",
			maxResults: data.numVideos,
			q: data.searchTerm,
			key: TLYouTubeFeedHelper.apiKey
		},function(data) {
			TLYouTubeFeedHelper.feedLoadFunc({
				callback: callback,
				feedData: data	
			}); 
		});
		
		
		
		/*$.getJSON("http://gdata.youtube.com/feeds/api/videos"+"?callback=?", {
			"q": data.searchTerm,
			"start-index": data.startIndex,
			"max-results": data.numVideos,
			"v": 2,
			"alt": "json",
			"orderby": "published"
		}, function(data) {
			TLYouTubeFeedHelper.feedLoadFunc({
				callback: callback,
				feedData: data	
			}); 
		});*/
	},
	feedLoadFunc: function(data) {
		var callback = data.callback;
		var feedData = data.feedData;
		if (!feedData || !feedData.items) {
			return;	
		}
		var returnObjs = TLYouTubeFeedHelper.processJSONFromArray({
			anArray: feedData.items	
		});
		callback({ itemObjs: returnObjs });	
	},
	processJSONFromArray: function(data) {
		var anArray = data.anArray;
		var returnObjs = new Array();
		var counter = 0;
		if (!anArray) {
			return [];	
		}
		$.each(anArray, function() {
			
			if (this.id.videoId) {
				this.id = this.id.videoId; 	
			}
			
			if (this.snippet.resourceId) {
				var videoId = this.snippet.resourceId.videoId;
				if (videoId) {
					this.id = videoId;
				}
			}
						
			var thumbnails = this.snippet.thumbnails;
			var thumbImage =  (thumbnails) ? thumbnails.medium || thumbnails["default"] : { url: TLYouTubeFeedHelper.defaultThumbImage }; 
			
			var aDate =  this.snippet.publishedAt;
			aDate = aDate.replace("T"," ").replace(".000Z","");
			
			var anObj = {
				id: this.id,
				owner: "",
				title: this.snippet.title,
				description: this.snippet.description,
				url: "https://www.youtube.com/watch?v="+this.id,
				image: thumbImage.url,
				ownername: "",
				date: AJKHelpers.dateFromMySQLDate({ dateString: aDate }),
				index: counter++
			}
			returnObjs.push(anObj);
		});		
		return returnObjs;
	}
}